@@ -4,8 +4,9 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) 2014 Anindya ROY (http://roy-a.github.io/)
-# Copyright (c) 2013-2014 Hervé BREDIN (http://herve.niderb.fr/)
+# Copyright (c) 2013-2014 CNRS
+#   - Anindya ROY (http://roy-a.github.io/)
+#   - Hervé BREDIN (http://herve.niderb.fr/)
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -29,6 +30,7 @@
 
 import re
 import urllib3
+import HTMLParser
 from pkg_resources import resource_filename
 from tvd import TFloating, TStart, TEnd, AnnotationGraph
 from tvd import Plugin
@@ -56,11 +58,15 @@ class TheBigBangTheory(Plugin):
         """
 
         http = urllib3.PoolManager()
+        h = HTMLParser.HTMLParser()
         r = http.request('GET', url)
         r = r.data
         r = re.sub('<script[^<]+</script>', '', r)
         r = re.sub('<style[^<]+</style>', '', r)
         r = re.sub('<div[^<]+</div>', '', r)
+        r = re.sub('<li>', '@EVENT', r) # Alternate way to detect event,
+                    # without depending on 'IXV.' etc.
+                    # -> Events are always items in a list.
         r = re.sub('<[^>]+>', '', r)
         r = r.split('\n')
 
@@ -74,7 +80,8 @@ class TheBigBangTheory(Plugin):
         start = 0
 
         for line in r:
-
+        
+            line = h.unescape(line) # Decode HTML code e.g. "don&#8217;t feed the .." to Unicode.
             if re.search('\A[ \t\n\r]*\Z', line):  # Empty line.
                 continue
 
@@ -86,7 +93,7 @@ class TheBigBangTheory(Plugin):
                 continue
 
             if start == 1:
-
+            # Check end of episode outline (or empty content: 'still to come').
                 if (
                     re.search(
                         '\A[ \t]*resources[ \t]*\Z',
@@ -103,48 +110,72 @@ class TheBigBangTheory(Plugin):
                     re.search(
                         '\A[ \t]*trivia[ \t]*\Z',
                         line,
+                        re.IGNORECASE) or
+                    re.search(
+                        '\A[ \t]*Still to come[ \t]*\Z',
+                        line,
                         re.IGNORECASE)
                 ):
                     break
 
-                # New location description.
-                if re.search('\A[ \t]*[IVXLCM]+[\.:]+', line):
+            # Lines to be ignored:
+            # ----------------------
+            # 'Titles and opening theme',
+            # 'Titles and credits'
+            # 'Opening themes and credits'
+            # 'Title and Opening Themes'
+            # 'Theme song and titles'
 
-                    # Finish the edge for previous location section.
-                    if t_event_prev:
-                        G.add_annotation(t_event_prev, t_location_prev, {})
+            # New location description.
+            #if re.search('\A[ \t]*[IVX]+[\.:]+', line): # DO NOT USE, exceptions to syntax exist.
+            if not re.search('@EVENT', line):
 
-                    location_ = re.sub(
-                        '\A[ \t]*[IVXLCM]+[\.:]+[ \t]*', '', line)
-                    t_location_start = TFloating()
-                    G.add_annotation(t_location_prev, t_location_start, {})
-                    t_location_stop = TFloating()
-                    G.add_annotation(
+                if (
+                    re.search('title', line, re.IGNORECASE) or
+                    re.search('credit', line, re.IGNORECASE) or
+                    re.search('theme', line, re.IGNORECASE)
+                ):
+                    continue # Assume it's 'Titles and opening theme' or something
+                     # similar. Ignore.
+
+                # Finish the edge for previous location section.
+                if t_event_prev:
+                    G.add_annotation(t_event_prev, t_location_prev, {})
+
+                location_ = re.sub(
+                    '\A[ \t]*[IVX]+[\.:]+[ \t]*', '', line) # Remove roman numeral.
+                t_location_start = TFloating()
+                G.add_annotation(t_location_prev, t_location_start, {})
+                t_location_stop = TFloating()
+                G.add_annotation(
                         t_location_start, t_location_stop,
                         {'location': location_}
-                    )
-                    t_location_prev = t_location_stop
-                    t_event_prev = t_location_start
+                )
+                t_location_prev = t_location_stop
+                t_event_prev = t_location_start
 
-                else:
+            else:
 
-                    event_ = ' '.join(line.split())
-                    t_event_start = TFloating()
-                    t_event_stop = TFloating()
-                    G.add_annotation(t_event_prev, t_event_start, {})
-                    G.add_annotation(
+                event_ = ' '.join(line.split())
+                event_ = re.sub('@EVENT', '', event_)
+                t_event_start = TFloating()
+                t_event_stop = TFloating()
+                G.add_annotation(t_event_prev, t_event_start, {})
+                G.add_annotation(
                         t_event_start, t_event_stop,
                         {'event': event_}
-                    )
-                    t_event_prev = t_event_stop
+                )
+                t_event_prev = t_event_stop
 
-        G.add_annotation(t_location_prev, t_episode_stop, {})
+        G.add_annotation(t_event_prev, t_location_prev, {}) # Finish event.
+        G.add_annotation(t_location_prev, t_episode_stop, {}) # Finish episode.
 
         return G
 
     def manual_transcript(self, url=None, episode=None, **kwargs):
 
         http = urllib3.PoolManager()
+        h = HTMLParser.HTMLParser()
         r = http.request('GET', url)
         r = r.data
         r = re.sub('<script[^<]+</script>', '', r)
@@ -161,6 +192,8 @@ class TheBigBangTheory(Plugin):
 
         for line in r:
 
+            #line = h.unescape(line) # Decode HTML code e.g. 
+                    # "don&#8217;t feed the .." to Unicode.
             # Empty line
             if re.search('\A[ \t\n\r]*\Z', line):
                 continue
@@ -192,7 +225,7 @@ class TheBigBangTheory(Plugin):
                 break
 
             speaker_ = re.match('\A\s*[^:\.]+\s*:', line)
-            # Comments e.G. '(They begin to fill out forms.)
+            # Comments e.g. '(They begin to fill out forms.)
             if speaker_ is None:
                 continue
 
@@ -213,6 +246,8 @@ class TheBigBangTheory(Plugin):
             )
             t_event_prev = t_event_stop
 
+        G.add_annotation(t_event_prev, t_location_prev, {})
+        G.add_annotation(t_location_prev, t_episode_stop, {})
         return G
 
 from ._version import get_versions
